
(*******************************
*  TmsStringDialog version 1.1 *
*  (c) 2007-2009 MarcaSoft     *
*******************************)
unit msDialogs;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, stdctrls,
  extctrls, buttons;


  {TmsCustomStringDialog}     

  Type
  TmsCustomStringDialog = class(TComponent)
  private
    { Private declarations }
    fForm: TForm;           //the form used for the dialog
    fLabel: TLabel;       //the label used to display prompt message
    fEdit: TEdit;          //the edit box used to input data
    fBtnOk: TBitBtn;
    fBtnCancel: TBitBtn;
    fOnShow: TNotifyEvent;
    fOnClose: TNotifyEvent;
    fEditKeyPress: TKeyPressEvent;
    fEditChange: TNotifyEvent;
    fEditKeyDown: TKeyEvent;
    fEditKeyUp: TKeyEvent;
    fCaption: string;       //the caption to appear in the caption bar of the dialog
    fEditText: string;      //the text to appear in the dialogs edit box when the dialog opens
    fPromptText: string;    //the text to be used to prompt the user to enter a string for example
                            // 'Please enter your string below'
    fPosition: TPosition;
    protected
    { Protected declarations }
    procedure SetSizePosition; virtual;  // Sets the size and position of the Dialog's edit control.
    procedure SetFormText; virtual;  // Sets the forms Caption, Prompt message, and Default text.
    Procedure fFormOnShow(sender: TObject); virtual;  

    //Override one or more of the following porcedures to handle changes to Dialog's edit control.
    Procedure fEditOnKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState); virtual;
    Procedure fEditOnKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState); virtual;
    Procedure fEditOnKeyPress(Sender: Tobject; var Key: char); virtual;
    Procedure fEditOnChange(Sender: TObject); virtual;
  public
    { Public declarations }
    constructor Create(AOwner:TComponent); override;
    destructor Destroy; override;
    Function Execute: boolean; virtual;
  published
    { Published declarations }
    Property Caption: string
      read fcaption write fcaption;
    Property Text: string
      read fEditText write fEditText;
    Property Prompt: string
      read fPromptText write fPromptText;
    Property Position: TPosition
      read fPosition write fPosition default poScreenCenter;
    Property OnShow: TnotifyEvent
      read fOnShow write fOnShow;
    Property OnClose: TnotifyEvent
      read fOnClose write fOnClose;
    Property OnKeyPress: TKeyPressEvent
      read fEditKeyPress write fEditKeyPress;
    Property OnChange: TNotifyEvent
      read fEditChange write fEditChange;
    Property OnKeyDown: TKeyEvent
      read fEditKeyDown write fEditKeyDown;
    Property OnKeyUp: TKeyEvent
      read fEditKeyUp write fEditKeyUp;
end;

  Procedure Register;


  {TmsStringDialog}

  type
  TmsStringDialogCharacterType = (sdcNumbers,sdcLetters,sdcBoth);
                            //used to set the edit box to allow only numerical characters,
                            //letters, or both

  TmsStringDialogType = (sddGeneral, sddFileName);
                            //used to determine the type of dialog

  TmsStringDialog = class(TmsCustomStringDialog)
  private
   { Private declarations }
    fPassChar: char;  //Character to be used for a password edit
    fAutoSelect: boolean;   //if text in edit box is to be automaticly selected
    fCase: TEditCharCase; //dialog's character case
    fCharType: TmsStringDialogCharacterType; //dialog's character type
    fDialogType: TmsStringDialogType;  //dialog type
    fMaxLength: integer;
    Procedure SetEditLength; //if the length of the prompt text > maxLength then then truncate the text
  protected
    { Protected declarations }
    Procedure fEditOnKeyPress(Sender: Tobject; var Key: char); override;
    procedure SetFormText; override;
  public
    { Public declarations }
    constructor Create(AOwner:TComponent); override;
    Function Execute: boolean; override;
  published
    { Published declarations }
    Property CharCase: TEditCharCase
      read fCase write fCase default ecNormal;
    Property MaxLength: integer
      read fMaxLength write fMaxLength default 0;
    Property CharType: TmsStringDialogCharacterType
      read fCharType write fCharType default sdcBoth;
    Property DialogType: TmsStringDialogType
      read fDialogType write fDialogType default sddGeneral;
    Property AutoSelect: boolean
      read fAutoSelect write fAutoSelect default true;
    Property PasswordChar: char
      read fPasschar write fPasschar default #0;
end;

implementation


{TmsCustomStringDialog}

constructor TmsCustomStringDialog.Create(AOwner:TComponent);
begin
  inherited Create(AOwner);
  fForm:=TForm.create(self);
  fLabel:=tLabel.create(self);
  fEdit:=TEdit.create(self);
  fBtnOk:=TBitBtn.create(self);
  fBtnCancel:=TBitBtn.create(self);
  fPosition:=poScreenCenter;
  fEdit.OnKeyPress:=fEditOnKeyPress;
  fEdit.OnChange:=fEditOnChange;
  fEdit.OnKeyDown:=fEditOnKeyDown;
  fEdit.OnKeyUp:=fEditOnKeyUp;
  with fForm do begin
    OnShow:=fFormOnShow;
    BorderStyle:=bsToolWindow;
    Caption:=fCaption;
    Height:=164;
    Width:=363;
    position:=fposition;
  end;
  with fLabel do begin
    Parent:=fForm;
    AutoSize:=True;
    WordWrap:=true;
    left:=7;
    top:=8;
    width:=338;
    caption:=fPromptText;
  end;
  with fEdit do begin
    parent:=fForm;
    width:=337;
    height:=21;
    text:=fEditText;
    left:=7;
    top:=fLabel.top+fLabel.height+10;
  end;
  fCaption:='String Dialog';
  fEditText:='';
  fPromptText:='Please enter text below.';
  fPosition:=poScreenCenter;
  with fBtnOk do begin
    parent:=fForm;
    kind:=bkOk;
    left:=168;
    top:=fEdit.top+fEdit.height+20;
  end;
  with fBtnCancel do begin
    parent:=fForm;
    kind:=bkCancel;
    left:=256;
    top:=fEdit.top+fEdit.height+20;
  end;
  with fForm do begin
    height:=fBtnOk.top+fBtnOk.height+20;
  end;
end;

procedure TmsCustomStringDialog.fEditOnKeyPress(Sender: TObject; var Key: Char);
begin
  if assigned (OnKeyPress) then
    OnKeyPress(Sender, Key);
end;

procedure TmsCustomStringDialog.fEditOnChange(Sender: TObject);
begin
  if assigned (OnChange) then
    OnChange(Sender);
end;

Procedure TmsCustomStringDialog.fEditOnKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  if assigned (OnKeyDown) then
    OnKeyDown(Sender, Key, Shift);
end;

Procedure TmsCustomStringDialog.fEditOnKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  if assigned (OnKeyUp) then
    OnKeyUp(Sender, Key, Shift);
end;

procedure TmsCustomStringDialog.SetFormText;
begin
  fEdit.text:=fEditText;
  fForm.caption:=fcaption;
  fLabel.caption:=fPromptText;
end;

procedure TmsCustomStringDialog.SetSizePosition;
begin
  fLabel.width:=337;
  fLabel.width:=337;
  fEdit.top:=fLabel.top+fLabel.height+10;
  fBtnOk.top:=fEdit.top+fEdit.height+20;
  fBtnCancel.top:=fEdit.top+fEdit.height+20;
  fForm.clientheight:=fBtnOk.top+fBtnOk.height+20;
  fForm.position:=fposition;
end;

Procedure TmsCustomStringDialog.fFormOnShow(sender: TObject);
begin
  fEdit.setfocus;
end;

function TmsCustomStringDialog.execute: boolean;
begin
  (* NOTICE!! When overriding this function, one must first make any changes to
     fForm or any of it's components before calling inherated! If inherated is first
     called fForm will be shown modal and any changes to the form or it's components will
     not me made untill after the form is closed by the user.*)

  if assigned(OnShow) then
    OnShow(Self);
  result:=true;
  SetFormText;
  SetSizePosition;
  fForm.showmodal;
  if fForm.modalresult=mrOk then begin
    fEditText:=fEdit.text;
  end else begin
    result:=false;
  end;
  if assigned(OnClose) then
    OnClose(self);
end;

destructor TmsCustomStringDialog.Destroy;
begin
  fLabel.free;
  fEdit.free;
  fBtnOk.free;
  fBtnCancel.free;
  fForm.free;
  inherited Destroy;
end;


{TmsStringDialog}


constructor TmsStringDialog.Create(AOwner:TComponent);
begin
  inherited Create(AOwner);
  fAutoSelect:=True;
  fcase:=ecNormal;
  fCharType:=sdcBoth;
  fDialogType:=sddGeneral;
  fMaxLength:=0;
  fPassChar:=#0;
end;

Procedure TmsStringDialog.fEditOnKeyPress(Sender: Tobject; var Key: char);
begin
  if key <> #8 then begin
    if (fCharType = sdcNumbers) then begin
      if (key <> '0')
      and (key <> '1')
      and (key <> '2')
      and (key <> '3')
      and (key <> '4')
      and (key <> '5')
      and (key <> '6')
      and (key <> '7')
      and (key <> '8')
      and (key <> '9')
      and (key <> '.')
      and (key <> '+')
      and (key <> '-')
      and (key <> '*')
      and (key <> '/')
      and (key <> '=')
      and (key <> '$')
      and (key <> '%')
      then
        key:=#0;
    end else if fCharType = sdcLetters then begin
      if (key = '0')
      or (key = '1')
      or (key = '2')
      or (key = '3')
      or (key = '4')
      or (key = '5')
      or (key = '6')
      or (key = '7')
      or (key = '8')
      or (key = '9')
      then
        key:=#0;
    end;
    if fDialogType = sddFileName then begin
      if (key ='\') or
      (key ='/') or
      (key =':') or
      (key ='*') or
      (key ='?') or
      (key ='"') or
      (key ='<') or
      (key ='>') or
      (key ='|') or
      (key ='.') then
         key:=#0;
    end;
  end;
  inherited fEditOnKeyPress(Sender, Key);
end;

Procedure TmsStringDialog.SetEditLength;
var
  key: char;
  x: integer;
  ClearText: boolean;
  s: string;
begin
  ClearText:= false;
  if (fCharType = sdcNumbers) then begin
    for x:= 1 to length(fEdit.text) do begin
      key:=fEdittext[x];
      if (key <> '0')
      and (key <> '1')
      and (key <> '2')
      and (key <> '3')
      and (key <> '4')
      and (key <> '5')
      and (key <> '6')
      and (key <> '7')
      and (key <> '8')
      and (key <> '9')
      and (key <> '.')
      and (key <> '+')
      and (key <> '-')
      and (key <> '*')
      and (key <> '/')
      and (key <> '=')
      and (key <> '$')
      and (key <> '%')
      then
        ClearText:=true;
    end;
  end else if fCharType = sdcLetters then begin
    for x:= 1 to length(fEdit.text) do begin
      key:=fEdittext[x];
      if (key = '0')
      or (key = '1')
      or (key = '2')
      or (key = '3')
      or (key = '4')
      or (key = '5')
      or (key = '6')
      or (key = '7')
      or (key = '8')
      or (key = '9')
      then
        ClearText:=true;
    end;
  end;
  if fDialogType = sddFileName then begin
    for x:= 1 to length(fEdit.text) do begin
      key:=fEdittext[x];
      if (key ='\') or
      (key ='/') or
      (key =':') or
      (key ='*') or
      (key ='?') or
      (key ='"') or
      (key ='<') or
      (key ='>') or
      (key ='|') or
      (key ='.') then
      ClearText:=true;
    end;
  end;
  if fCase=eclowercase then
    fEdit.text:=lowercase(fEditText)
  else if fCase=ecUppercase then
    fEdit.text:=uppercase(fEditText)
  else begin
    fEdit.text:=fEditText;
  end;
  if fMaxLength<>0 then begin
    if length(fEditText)>fMaxLength then begin
      s:=Copy(fEditText,1,fMaxLength);
      fEdit.text:=s;
    end;
  end;
  if ClearText then
    fEdit.text:='';
end;

procedure TmsStringDialog.SetFormText;
begin
  fForm.caption:=fcaption;
  fLabel.caption:=fPromptText;
end;

function TmsStringDialog.execute: boolean;
begin
  fEdit.text:=fedittext;
  fEdit.charcase:=fcase;
  fEdit.autoselect:=fAutoSelect;
  fEdit.maxLength:=fMaxLength;
  fEdit.PasswordChar:=fPassChar;
  SetEditLength;
  if not fAutoSelect then begin
    fEdit.selstart:=0;
    fEdit.sellength:=0;
  end;
  result:= inherited execute;
end;

procedure Register;
begin
  RegisterComponents('MarcaSoft', [TmsStringDialog]);
end;

end.
